@@ -1,11 +1,14 @@
+# frozen_string_literal: true
+
 require 'vagrant/action/builder'
 require 'log4r'
 
 module VagrantPlugins
   module ProviderLibvirt
     module Action
-      # Include the built-in modules so we can use them as top-level things.
+      # Include the built-in & general modules so we can use them as top-level things.
       include Vagrant::Action::Builtin
+      include Vagrant::Action::General
       @logger = Log4r::Logger.new('vagrant_libvirt::action')
 
       # remove image from Libvirt storage pool
@@ -136,11 +139,21 @@ module VagrantPlugins
               b3.use ResumeDomain if env2[:result]
             end
 
+            # only perform shutdown if VM is running
             b2.use Call, IsRunning do |env2, b3|
               next unless env2[:result]
 
-              # VM is running, halt it.
-              b3.use HaltDomain
+              b3.use StartShutdownTimer
+              b3.use Call, GracefulHalt, :shutoff, :running do |env3, b4|
+                if !env3[:result]
+                  b4.use Call, ShutdownDomain, :shutoff, :running do |env4, b5|
+                    if !env4[:result]
+                       b5.use HaltDomain
+                    end
+                  end
+                end
+              end
+
             end
           end
         end
@@ -167,7 +180,18 @@ module VagrantPlugins
       def self.action_package
         Vagrant::Action::Builder.new.tap do |b|
           b.use ConfigValidate
-          b.use PackageDomain
+          b.use Call, IsCreated do |env, b2|
+            unless env[:result]
+              b2.use MessageNotCreated
+              next
+            end
+
+            b2.use PackageSetupFolders
+            b2.use PackageSetupFiles
+            b2.use action_halt
+            b2.use Package
+            b2.use PackageDomain
+          end
         end
       end
 
@@ -209,14 +233,12 @@ module VagrantPlugins
           b.use ConfigValidate
           b.use Call, IsCreated do |env, b2|
             unless env[:result]
-              b2.use MessageNotCreated
-              next
+              raise Vagrant::Errors::VMNotCreatedError
             end
 
             b2.use Call, IsRunning do |env2, b3|
               unless env2[:result]
-                b3.use MessageNotRunning
-                next
+                raise Vagrant::Errors::VMNotRunningError
               end
 
               b3.use SSHExec
@@ -306,14 +328,12 @@ module VagrantPlugins
           b.use ConfigValidate
           b.use Call, IsCreated do |env, b2|
             unless env[:result]
-              b2.use MessageNotCreated
-              next
+              raise Vagrant::Errors::VMNotCreatedError
             end
 
             b2.use Call, IsRunning do |env2, b3|
               unless env2[:result]
-                b3.use MessageNotRunning
-                next
+                raise Vagrant::Errors::VMNotRunningError
               end
 
               b3.use SSHRun
@@ -334,6 +354,8 @@ module VagrantPlugins
       autoload :ForwardPorts, action_root.join('forward_ports')
       autoload :ClearForwardedPorts, action_root.join('forward_ports')
       autoload :HaltDomain, action_root.join('halt_domain')
+      autoload :StartShutdownTimer, action_root.join('shutdown_domain')
+      autoload :ShutdownDomain, action_root.join('shutdown_domain')
       autoload :HandleBoxImage, action_root.join('handle_box_image')
       autoload :HandleStoragePool, action_root.join('handle_storage_pool')
       autoload :RemoveLibvirtImage, action_root.join('remove_libvirt_image')
@@ -366,6 +388,9 @@ module VagrantPlugins
       autoload :WaitTillUp, action_root.join('wait_till_up')
       autoload :PrepareNFSValidIds, action_root.join('prepare_nfs_valid_ids')
 
+      autoload :Package, 'vagrant/action/general/package'
+      autoload :PackageSetupFiles, 'vagrant/action/general/package_setup_files'
+      autoload :PackageSetupFolders, 'vagrant/action/general/package_setup_folders'
       autoload :SSHRun, 'vagrant/action/builtin/ssh_run'
       autoload :HandleBox, 'vagrant/action/builtin/handle_box'
       autoload :SyncedFolders, 'vagrant/action/builtin/synced_folders'
