@@ -1,3 +1,5 @@
+# frozen_string_literal: true
+
 require 'log4r'
 
 module VagrantPlugins
@@ -76,9 +78,11 @@ module VagrantPlugins
                              else
                                "passwd='#{config.graphics_passwd}'"
                               end
+          @graphics_gl = config.graphics_gl
           @video_type = config.video_type
           @sound_type = config.sound_type
           @video_vram = config.video_vram
+          @video_accel3d = config.video_accel3d
           @keymap = config.keymap
           @kvm_hidden = config.kvm_hidden
 
@@ -169,6 +173,20 @@ module VagrantPlugins
             storage_prefix = get_disk_storage_prefix(env, @storage_pool_name)
           end
 
+          @serials = config.serials
+
+          @serials.each do |serial|
+            next unless serial[:source] && serial[:source][:path]
+
+            dir = File.dirname(serial[:source][:path])
+            begin
+              FileUtils.mkdir_p(dir)
+            rescue ::Errno::EACCES
+              raise Errors::SerialCannotCreatePathError,
+                    path: dir
+            end
+          end
+
           @disks.each do |disk|
             disk[:path] ||= _disk_name(@name, disk)
 
@@ -266,7 +284,7 @@ module VagrantPlugins
           end
           env[:ui].info(" -- Storage pool:      #{@storage_pool_name}")
           @domain_volumes.each do |volume|
-            env[:ui].info(" -- Image(#{volume[:device]}):     #{volume[:path]}, #{volume[:virtual_size]}G")
+            env[:ui].info(" -- Image(#{volume[:device]}):     #{volume[:path]}, #{volume[:virtual_size].to_GB}G")
           end
 
           if not @disk_driver_opts.empty?
@@ -283,6 +301,7 @@ module VagrantPlugins
           env[:ui].info(" -- Graphics Password: #{@graphics_passwd.empty? ? 'Not defined' : 'Defined'}")
           env[:ui].info(" -- Video Type:        #{@video_type}")
           env[:ui].info(" -- Video VRAM:        #{@video_vram}")
+          env[:ui].info(" -- Video 3D accel:    #{@video_accel3d}")
           env[:ui].info(" -- Sound Type:	#{@sound_type}")
           env[:ui].info(" -- Keymap:            #{@keymap}")
           env[:ui].info(" -- TPM Backend:       #{@tpm_type}")
@@ -377,6 +396,13 @@ module VagrantPlugins
             env[:ui].info(" -- smartcard device:  mode=#{@smartcard_dev[:mode]}, type=#{@smartcard_dev[:type]}")
           end
 
+          @serials.each_with_index do |serial, port|
+            if serial[:source]
+              env[:ui].info(" -- SERIAL(COM#{port}:       redirect to #{serial[:source][:path]}")
+              env[:ui].warn(I18n.t('vagrant_libvirt.warnings.creating_domain_console_access_disabled'))
+            end
+          end
+
           unless @qemu_args.empty?
             env[:ui].info(' -- Command line args: ')
             @qemu_args.each do |arg|
@@ -398,9 +424,14 @@ module VagrantPlugins
           # Create Libvirt domain.
           # Is there a way to tell fog to create new domain with already
           # existing volume? Use domain creation from template..
+          xml = to_xml('domain')
+          @logger.debug {
+            "Creating Domain with XML:\n#{xml}"
+          }
+
           begin
             server = env[:machine].provider.driver.connection.servers.create(
-              xml: to_xml('domain')
+              xml: xml
             )
           rescue Fog::Errors::Error => e
             raise Errors::FogCreateServerError, error_message: e.message
