@@ -1,3 +1,5 @@
+# frozen_string_literal: true
+
 require 'log4r'
 
 module VagrantPlugins
@@ -39,7 +41,7 @@ module VagrantPlugins
             @backing_file = box_volume.path
 
             # Virtual size of image. Take value worked out by HandleBoxImage
-            @capacity = env[:box_volumes][index][:virtual_size] # G
+            @capacity = env[:box_volumes][index][:virtual_size].to_B # Byte
 
             # Create new volume from xml template. Fog currently doesn't support
             # volume snapshots directly.
@@ -47,7 +49,7 @@ module VagrantPlugins
               xml = Nokogiri::XML::Builder.new do |xml|
                 xml.volume do
                   xml.name(@name)
-                  xml.capacity(@capacity, unit: 'G')
+                  xml.capacity(@capacity, unit: 'B')
                   xml.target do
                     xml.format(type: 'qcow2')
                     xml.permissions do
@@ -76,6 +78,9 @@ module VagrantPlugins
               else
                   pool_name = config.storage_pool_name
               end
+              @logger.debug {
+                "Creating Volume with XML:\n#{xml}"
+              }
               @logger.debug "Using pool #{pool_name} for base box snapshot"
               domain_volume = env[:machine].provider.driver.connection.volumes.create(
                 xml: xml,
