@@ -1,3 +1,5 @@
+# frozen_string_literal: true
+
 require 'log4r'
 require 'vagrant/util/network_ip'
 require 'vagrant/util/scoped_hash_override'
@@ -160,7 +162,7 @@ module VagrantPlugins
             if @interface_network[:created]
               # Just check for mismatch error here - if name and ip from
               # config match together.
-              if @options[:network_name] != @interface_network[:name]
+              if @options[:network_name] != @interface_network[:name] and @qemu_use_agent == false
                 raise Errors::NetworkNameAndAddressMismatch,
                       ip_address:   @options[:ip],
                       network_name: @options[:network_name]
@@ -206,6 +208,9 @@ module VagrantPlugins
 
             # Create a private network.
             create_private_network(env)
+            write_created_network(env)
+          else
+            write_created_network(env) unless @options[:always_destroy] == false
           end
         end
 
@@ -245,6 +250,9 @@ module VagrantPlugins
 
             # Create a private network.
             create_private_network(env)
+            write_created_network(env)
+          else
+            write_created_network(env) unless @options[:always_destroy] == false
           end
         end
 
@@ -271,6 +279,9 @@ module VagrantPlugins
 
             # Create a private network.
             create_private_network(env)
+            write_created_network(env)
+          else
+            write_created_network(env) unless @options[:always_destroy] == false
           end
         end
 
@@ -312,8 +323,7 @@ module VagrantPlugins
 
           if @options[:dhcp_enabled]
             # Find out DHCP addresses pool range.
-            network_address = "#{@interface_network[:network_address]}/"
-            network_address << (@interface_network[:netmask]).to_s
+            network_address = "#{@interface_network[:network_address]}/#{(@interface_network[:netmask]).to_s}"
             net = @interface_network[:network_address] ? IPAddr.new(network_address) : nil
 
             # First is address of network, second is gateway (by default).
@@ -336,19 +346,24 @@ module VagrantPlugins
           @network_domain_name = @options[:domain_name]
 
           begin
+            xml = to_xml('private_network')
+            @logger.debug {
+              "Creating private network with XML:\n#{xml}"
+            }
             @interface_network[:libvirt_network] = \
-              @libvirt_client.define_network_xml(to_xml('private_network'))
+              @libvirt_client.define_network_xml(xml)
             @logger.debug 'created network'
           rescue => e
             raise Errors::CreateNetworkError, error_message: e.message
           end
+        end
 
+        def write_created_network(env)
           created_networks_file = env[:machine].data_dir + 'created_networks'
 
-          message = 'Saving information about created network '
-          message << "#{@interface_network[:name]}, "
-          message << "UUID=#{@interface_network[:libvirt_network].uuid} "
-          message << "to file #{created_networks_file}."
+          message = 'Saving information about created network ' \
+            "#{@interface_network[:name]}, UUID=#{@interface_network[:libvirt_network].uuid} " \
+            "to file #{created_networks_file}."
           @logger.info(message)
 
           File.open(created_networks_file, 'a') do |file|
