@@ -1,3 +1,5 @@
+# frozen_string_literal: true
+
 require 'cgi'
 
 require 'vagrant'
@@ -5,10 +7,10 @@ require 'vagrant'
 class Numeric
   Alphabet = ('a'..'z').to_a
   def vdev
-    s = ''
+    s = String.new
     q = self
     (q, r = (q - 1).divmod(26)) && s.prepend(Alphabet[r]) until q.zero?
-    'vd' + s
+    "vd#{s}"
   end
 end
 
@@ -117,8 +119,10 @@ module VagrantPlugins
       attr_accessor :graphics_port
       attr_accessor :graphics_passwd
       attr_accessor :graphics_ip
+      attr_accessor :graphics_gl
       attr_accessor :video_type
       attr_accessor :video_vram
+      attr_accessor :video_accel3d
       attr_accessor :keymap
       attr_accessor :kvm_hidden
       attr_accessor :sound_type
@@ -191,6 +195,12 @@ module VagrantPlugins
       # Use QEMU session instead of system
       attr_accessor :qemu_use_session
 
+      # Use QEMU Agent to get ip address
+      attr_accessor :qemu_use_agent
+
+      # serial consoles
+      attr_accessor :serials
+
       def initialize
         @uri               = UNSET_VALUE
         @driver            = UNSET_VALUE
@@ -262,8 +272,10 @@ module VagrantPlugins
         @graphics_port     = UNSET_VALUE
         @graphics_ip       = UNSET_VALUE
         @graphics_passwd   = UNSET_VALUE
+        @graphics_gl       = UNSET_VALUE
         @video_type        = UNSET_VALUE
         @video_vram        = UNSET_VALUE
+        @video_accel3d     = UNSET_VALUE
         @sound_type        = UNSET_VALUE
         @keymap            = UNSET_VALUE
         @kvm_hidden        = UNSET_VALUE
@@ -330,6 +342,11 @@ module VagrantPlugins
         @qemu_env          = UNSET_VALUE
 
         @qemu_use_session  = UNSET_VALUE
+
+        # Use Qemu agent to get ip address
+        @qemu_use_agent  = UNSET_VALUE
+
+        @serials           = []
       end
 
       def boot(device)
@@ -352,7 +369,7 @@ module VagrantPlugins
         # hda - hdc
         curr = 'a'.ord
         while curr <= 'd'.ord
-          dev = 'hd' + curr.chr
+          dev = "hd#{curr.chr}"
           if exist[dev]
             curr += 1
             next
@@ -555,7 +572,7 @@ module VagrantPlugins
         end
 
         @usbctl_dev[:model] = options[:model]
-        @usbctl_dev[:ports] = options[:ports]
+        @usbctl_dev[:ports] = options[:ports] if options[:ports]
       end
 
       def usb(options = {})
@@ -697,6 +714,20 @@ module VagrantPlugins
         @qemu_env.merge!(options)
       end
 
+      def serial(options={})
+        options = {
+          :type => "pty",
+          :source => nil,
+        }.merge(options)
+
+        serial = {
+          :type => options[:type],
+          :source => options[:source],
+        }
+
+        @serials << serial
+      end
+
       def _default_uri
         # Determine if any settings except driver provided explicitly, if not
         # and the LIBVIRT_DEFAULT_URI var is set, use that.
@@ -745,24 +776,24 @@ module VagrantPlugins
         if @connect_via_ssh == true
           finalize_id_ssh_key_file
 
-          uri << '+ssh://'
-          uri << @username + '@' if @username && @username != UNSET_VALUE
+          uri += '+ssh://'
+          uri += "#{@username}@" if @username && @username != UNSET_VALUE
 
-          uri << ( @host && @host != UNSET_VALUE ? @host : 'localhost' )
+          uri += (@host && @host != UNSET_VALUE ? @host : 'localhost')
 
           params['no_verify'] = '1'
           params['keyfile'] = @id_ssh_key_file if @id_ssh_key_file
         else
-          uri << '://'
-          uri << @host if @host && @host != UNSET_VALUE
+          uri += '://'
+          uri += @host if @host && @host != UNSET_VALUE
         end
 
-        uri << virt_path
+        uri += virt_path
 
         # set path to Libvirt socket
         params['socket'] = @socket if @socket
 
-        uri << "?" + params.map{|pair| pair.join('=')}.join('&') if !params.empty?
+        uri += '?' + params.map { |pair| pair.join('=') }.join('&') unless params.empty?
         uri
       end
 
@@ -809,7 +840,6 @@ module VagrantPlugins
         @management_network_pci_slot = nil if @management_network_pci_slot == UNSET_VALUE
         @management_network_domain = nil if @management_network_domain == UNSET_VALUE
         @management_network_mtu = nil if @management_network_mtu == UNSET_VALUE
-        @system_uri      = 'qemu:///system' if @system_uri == UNSET_VALUE
 
         # Domain specific settings.
         @title = '' if @title == UNSET_VALUE
@@ -863,6 +893,8 @@ module VagrantPlugins
         @graphics_ip = '127.0.0.1' if @graphics_ip == UNSET_VALUE
         @video_type = 'cirrus' if @video_type == UNSET_VALUE
         @video_vram = 9216 if @video_vram == UNSET_VALUE
+        @video_accel3d = false if @video_accel3d == UNSET_VALUE
+        @graphics_gl = @video_accel3d if @graphics_gl == UNSET_VALUE
         @sound_type = nil if @sound_type == UNSET_VALUE
         @keymap = 'en-us' if @keymap == UNSET_VALUE
         @kvm_hidden = false if @kvm_hidden == UNSET_VALUE
@@ -907,9 +939,6 @@ module VagrantPlugins
         # Watchdog device
         @watchdog_dev = {} if @watchdog_dev == UNSET_VALUE
 
-        # USB controller
-        @usbctl_dev = {} if @usbctl_dev == UNSET_VALUE
-
         # USB device passthrough
         @usbs = [] if @usbs == UNSET_VALUE
 
@@ -917,6 +946,11 @@ module VagrantPlugins
         @redirdevs = [] if @redirdevs == UNSET_VALUE
         @redirfilters = [] if @redirfilters == UNSET_VALUE
 
+        # USB controller
+        if @usbctl_dev == UNSET_VALUE
+          @usbctl_dev = if !@usbs.empty? or !@redirdevs.empty? then {:model => 'qemu-xhci'} else {} end
+        end
+
         # smartcard device
         @smartcard_dev = {} if @smartcard_dev == UNSET_VALUE
 
@@ -934,6 +968,10 @@ module VagrantPlugins
 
         # Additional QEMU commandline environment variables
         @qemu_env = {} if @qemu_env == UNSET_VALUE
+
+        @qemu_use_agent = false if @qemu_use_agent == UNSET_VALUE
+
+        @serials = [{:type => 'pty', :source => nil}] if @serials == []
       end
 
       def validate(machine)
@@ -947,19 +985,38 @@ module VagrantPlugins
           end
         end
 
+        unless @qemu_use_agent == true || @qemu_use_agent == false
+          errors << "libvirt.qemu_use_agent must be a boolean."
+        end
+
+        if @qemu_use_agent == true
+          # if qemu agent is used to optain domain ip configuration, at least
+          # one qemu channel has to be configured. As there are various options,
+          # error out and leave configuration to the user
+          unless machine.provider_config.channels.any? { |channel| channel[:target_name].start_with?("org.qemu.guest_agent") }
+            errors << "qemu agent option enabled, but no qemu agent channel configured: please add at least one qemu agent channel to vagrant config"
+          end
+        end
+
+
         machine.provider_config.disks.each do |disk|
           if disk[:path] && (disk[:path][0] == '/')
             errors << "absolute volume paths like '#{disk[:path]}' not yet supported"
           end
         end
 
+        machine.provider_config.serials.each do |serial|
+          if serial[:source] and serial[:source][:path].nil?
+            errors << "serial :source requires :path to be defined"
+          end
+        end
+
         machine.config.vm.networks.each do |_type, opts|
           if opts[:mac]
-            opts[:mac].downcase!
-            if opts[:mac] =~ /\A([0-9a-f]{12})\z/
+            if opts[:mac] =~ /\A([0-9a-fA-F]{12})\z/
               opts[:mac] = opts[:mac].scan(/../).join(':')
             end
-            unless opts[:mac] =~ /\A([0-9a-f]{2}:){5}([0-9a-f]{2})\z/
+            unless opts[:mac] =~ /\A([0-9a-fA-F]{2}:){5}([0-9a-fA-F]{2})\z/
               errors << "Configured NIC MAC '#{opts[:mac]}' is not in 'xx:xx:xx:xx:xx:xx' or 'xxxxxxxxxxxx' format"
             end
           end
@@ -995,6 +1052,10 @@ module VagrantPlugins
           c = qemu_env != UNSET_VALUE ? qemu_env.dup : {}
           c.merge!(other.qemu_env) if other.qemu_env != UNSET_VALUE
           result.qemu_env = c
+
+          s = serials.dup
+          s += other.serials
+          result.serials = s
         end
       end
 
@@ -1004,6 +1065,10 @@ module VagrantPlugins
         # Parse uri to extract individual components
         uri = _parse_uri(@uri)
 
+        system_uri = uri.dup
+        system_uri.path = '/system'
+        @system_uri = system_uri.to_s if @system_uri == UNSET_VALUE
+
         # only set @connect_via_ssh if not explicitly to avoid overriding
         # and allow an error to occur if the @uri and @connect_via_ssh disagree
         @connect_via_ssh = uri.scheme.include? "ssh" if @connect_via_ssh == UNSET_VALUE
@@ -1017,10 +1082,12 @@ module VagrantPlugins
           end
         end
 
-        # Extract host and username values from uri if provided, otherwise nil
+        # Extract host values from uri if provided, otherwise nil
         @host = uri.host
         @port = uri.port
-        @username = uri.user
+        # only override username if there is a value provided
+        @username = nil if @username == UNSET_VALUE
+        @username = uri.user if uri.user
         if uri.query
           params = CGI.parse(uri.query)
           @id_ssh_key_file = params['keyfile'].first if params.has_key?('keyfile')
@@ -1032,7 +1099,7 @@ module VagrantPlugins
       def resolve_ssh_key_file(key_file)
         # set ssh key for access to Libvirt host
         # if no slash, prepend $HOME/.ssh/
-        key_file.prepend("#{ENV['HOME']}/.ssh/") if key_file && key_file !~ /\A\//
+        key_file = "#{ENV['HOME']}/.ssh/#{key_file}" if key_file && key_file !~ /\A\//
 
         key_file
       end
@@ -1061,17 +1128,17 @@ module VagrantPlugins
         if @connect_via_ssh
           if @proxy_command == UNSET_VALUE
             proxy_command = "ssh '#{@host}' "
-            proxy_command << "-p #{@port} " if @port
-            proxy_command << "-l '#{@username}' " if @username
-            proxy_command << "-i '#{@id_ssh_key_file}' " if @id_ssh_key_file
-            proxy_command << '-W %h:%p'
+            proxy_command += "-p #{@port} " if @port
+            proxy_command += "-l '#{@username}' " if @username
+            proxy_command += "-i '#{@id_ssh_key_file}' " if @id_ssh_key_file
+            proxy_command += '-W %h:%p'
           else
             inputs = { host: @host }
             inputs << { port: @port } if @port
             inputs[:username] = @username if @username
             inputs[:id_ssh_key_file] = @id_ssh_key_file if @id_ssh_key_file
 
-            proxy_command = @proxy_command
+            proxy_command = String.new(@proxy_command)
             # avoid needing to escape '%' symbols
             inputs.each do |key, value|
               proxy_command.gsub!("{#{key}}", value)
