@@ -1,17 +1,22 @@
+# frozen_string_literal: true
+
 require 'fog/libvirt'
 require 'libvirt'
 require 'log4r'
+require 'json'
 
 module VagrantPlugins
   module ProviderLibvirt
     class Driver
-      # store the connection at the process level
+      # store the connection at the instance level as this will be per
+      # thread and allows for individual machines to use different
+      # connection settings.
       #
       # possibly this should be a connection pool using the connection
-      # settings as a key to allow per machine connection attributes
-      # to be used.
-      @@connection = nil
-      @@system_connection = nil
+      # settings as a key to allow identical connections to be reused
+      # across machines.
+      @connection = nil
+      @system_connection = nil
 
       def initialize(machine)
         @logger = Log4r::Logger.new('vagrant_libvirt::driver')
@@ -21,7 +26,7 @@ module VagrantPlugins
       def connection
         # If already connected to Libvirt, just use it and don't connect
         # again.
-        return @@connection if @@connection
+        return @connection if @connection
 
         # Get config options for Libvirt provider.
         config = @machine.provider_config
@@ -40,24 +45,24 @@ module VagrantPlugins
 
         @logger.info("Connecting to Libvirt (#{uri}) ...")
         begin
-          @@connection = Fog::Compute.new(conn_attr)
+          @connection = Fog::Compute.new(conn_attr)
         rescue Fog::Errors::Error => e
           raise Errors::FogLibvirtConnectionError,
                 error_message: e.message
         end
 
-        @@connection
+        @connection
       end
 
       def system_connection
         # If already connected to Libvirt, just use it and don't connect
         # again.
-        return @@system_connection if @@system_connection
+        return @system_connection if @system_connection
 
         config = @machine.provider_config
 
-        @@system_connection = Libvirt::open_read_only(config.system_uri)
-        @@system_connection
+        @system_connection = Libvirt::open_read_only(config.system_uri)
+        @system_connection
       end
 
       def get_domain(machine)
@@ -93,6 +98,12 @@ module VagrantPlugins
       end
 
       def get_domain_ipaddress(machine, domain)
+        # attempt to get ip address from qemu agent
+        if @machine.provider_config.qemu_use_agent == true
+          @logger.info('Get IP via qemu agent')
+          return get_ipaddress_from_qemu_agent(domain, machine.id)
+        end
+
         if @machine.provider_config.qemu_use_session
           return get_ipaddress_from_system domain.mac
         end
@@ -126,7 +137,17 @@ module VagrantPlugins
         # TODO: terminated no longer appears to be a valid fog state, remove?
         return :not_created if domain.nil? || domain.state.to_sym == :terminated
 
-        domain.state.tr('-', '_').to_sym
+        state = domain.state.tr('-', '_').to_sym
+        if state == :running
+          begin
+            get_domain_ipaddress(machine, domain)
+          rescue Fog::Errors::TimeoutError => e
+            @logger.debug("Machine #{machine.id} running but no IP address available: #{e}.")
+            return :inaccessible
+          end
+        end
+
+        return state
       end
 
       private
@@ -144,6 +165,40 @@ module VagrantPlugins
         ip_address
       end
 
+      def get_ipaddress_from_qemu_agent(domain, machine_id)
+        ip_address = nil
+        addresses = nil
+        libvirt_domain = connection.client.lookup_domain_by_uuid(machine_id)
+        begin
+          response = libvirt_domain.qemu_agent_command('{"execute":"guest-network-get-interfaces"}', timeout=10)
+          @logger.debug("Got Response from qemu agent")
+          @logger.debug(response)
+          addresses = JSON.parse(response)
+        rescue => e
+          @logger.debug("Unable to receive IP via qemu agent: [%s]" % e.message)
+        end
+
+        unless addresses.nil?
+          addresses["return"].each{ |interface|
+            if domain.mac.downcase == interface["hardware-address"].downcase
+              @logger.debug("Found mathing interface: [%s]" % interface["name"])
+              if interface.has_key?("ip-addresses")
+                interface["ip-addresses"].each{ |ip|
+                  # returning ipv6 addresses might break windows guests because
+                  # winrm cant handle connection, winrm fails with "invalid uri"
+                  if ip["ip-address-type"] == "ipv4"
+                    ip_address = ip["ip-address"]
+                    @logger.debug("Return IP: [%s]" % ip_address)
+                    break
+                  end
+                  }
+              end
+            end
+          }
+        end
+        ip_address
+      end
+
       def get_ipaddress_from_domain(domain)
         ip_address = nil
         domain.wait_for(2) do
