@@ -1,3 +1,5 @@
+# frozen_string_literal: true
+
 require 'vagrant'
 
 module VagrantPlugins
@@ -7,6 +9,10 @@ module VagrantPlugins
         error_namespace('vagrant_libvirt.errors')
       end
 
+      class CallChainError < VagrantLibvirtError
+        error_key(:call_chain_error)
+      end
+
       # package not supported
       class PackageNotSupported < VagrantLibvirtError
         error_key(:package_not_supported)
@@ -175,6 +181,10 @@ module VagrantPlugins
       class DeleteSnapshotError < VagrantLibvirtError
         error_key(:delete_snapshot_error)
       end
+
+      class SerialCannotCreatePathError < VagrantLibvirtError
+        error_key(:serial_cannot_create_path_error)
+      end
     end
   end
 end
