@@ -1,3 +1,5 @@
+# frozen_string_literal: true
+
 require 'ipaddr'
 require 'nokogiri'
 require 'vagrant/util/network_ip'
@@ -18,6 +20,7 @@ module VagrantPlugins
 
         def configured_networks(env, logger)
           qemu_use_session = env[:machine].provider_config.qemu_use_session
+          qemu_use_agent = env[:machine].provider_config.qemu_use_agent
           management_network_device = env[:machine].provider_config.management_network_device
           management_network_name = env[:machine].provider_config.management_network_name
           management_network_address = env[:machine].provider_config.management_network_address
@@ -116,7 +119,8 @@ module VagrantPlugins
                             IPAddr.new(options[:network_address]).get_mask :
                             '255.255.255.0',
               dhcp_enabled: true,
-              forward_mode: 'nat'
+              forward_mode: 'nat',
+              always_destroy: true
             }.merge(options)
 
             if options[:type].to_s == 'dhcp' && options[:ip].nil?
