@@ -1,3 +1,5 @@
+# frozen_string_literal: true
+
 require 'support/binding_proc'
 
 require 'spec_helper'
@@ -190,14 +192,14 @@ describe VagrantPlugins::ProviderLibvirt::Config do
         ],
         [ # when username explicitly set without ssh
           {:username => 'my_user' },
-          {:uri => 'qemu:///system'},
+          {:uri => 'qemu:///system', :username => 'my_user'},
           {
             :env => {'LIBVIRT_DEFAULT_URI' => 'qemu://session'},
           }
         ],
         [ # when username explicitly set with host but without ssh
           {:username => 'my_user', :host => 'remote'},
-          {:uri => 'qemu://remote/system'},
+          {:uri => 'qemu://remote/system', :username => 'my_user'},
           {
             :env => {'LIBVIRT_DEFAULT_URI' => 'qemu://session'},
           }
@@ -336,6 +338,54 @@ describe VagrantPlugins::ProviderLibvirt::Config do
       end
     end
 
+    context '@system_uri' do
+      [
+        # system uri
+        [ # transport and hostname
+          {:uri => "qemu+ssh://localhost/session"},
+          {:uri => "qemu+ssh://localhost/session", :system_uri => "qemu+ssh://localhost/system"},
+        ],
+        [ # explicitly set
+          {:qemu_use_session => true, :system_uri => "custom://remote/system"},
+          {:uri => "qemu:///session", :system_uri => "custom://remote/system"},
+        ],
+      ].each do |inputs, outputs, options|
+        opts = {}
+        opts.merge!(options) if options
+
+        it "should handle inputs #{inputs} with env (#{opts[:env]})" do
+          # allow some of these to fail for now if marked as such
+          if !opts[:allow_failure].nil?
+            pending(opts[:allow_failure])
+          end
+
+          if !opts[:setup].nil?
+            opts[:setup].apply_binding(binding)
+          end
+
+          inputs.each do |k, v|
+            subject.instance_variable_set("@#{k}", v)
+          end
+
+          if !opts[:env].nil?
+            opts[:env].each do |k, v|
+              fake_env[k] = v
+            end
+          end
+
+          subject.finalize!
+
+          # ensure failed output indicates which settings are incorrect in the failed test
+          got = subject.instance_variables.each_with_object({}) do |name, hash|
+            if outputs.key?(name.to_s[1..-1].to_sym)
+              hash["#{name.to_s[1..-1]}".to_sym] =subject.instance_variable_get(name)
+            end
+          end
+          expect(got).to eq(outputs)
+        end
+      end
+    end
+
     context '@proxy_command' do
       before(:example) do
         stub_const("ENV", fake_env)
@@ -433,6 +483,54 @@ describe VagrantPlugins::ProviderLibvirt::Config do
         end
       end
     end
+
+    context '@usbctl_dev' do
+      it 'should be empty by default' do
+        subject.finalize!
+
+        expect(subject.usbctl_dev).to eq({})
+      end
+
+      context 'when usb devices added' do
+        it 'should inject a default controller' do
+          subject.usb :vendor => '0x1234', :product => '0xabcd'
+
+          subject.finalize!
+
+          expect(subject.usbctl_dev).to eq({:model => 'qemu-xhci'})
+        end
+
+        context 'when user specified a controller' do
+          it 'should retain the user setting' do
+            subject.usb :vendor => '0x1234', :product => '0xabcd'
+            subject.usb_controller :model => 'pii3-uchi'
+
+            subject.finalize!
+            expect(subject.usbctl_dev).to eq({:model => 'pii3-uchi'})
+          end
+        end
+      end
+
+      context 'when redirdevs entries added' do
+        it 'should inject a default controller' do
+          subject.redirdev :type => 'spicevmc'
+
+          subject.finalize!
+
+          expect(subject.usbctl_dev).to eq({:model => 'qemu-xhci'})
+        end
+
+        context 'when user specified a controller' do
+          it 'should retain the user setting' do
+            subject.redirdev :type => 'spicevmc'
+            subject.usb_controller :model => 'pii3-uchi'
+
+            subject.finalize!
+            expect(subject.usbctl_dev).to eq({:model => 'pii3-uchi'})
+          end
+        end
+      end
+    end
   end
 
   def assert_invalid
