@@ -1,3 +1,5 @@
+# frozen_string_literal: true
+
 require 'support/sharedcontext'
 
 require 'vagrant-libvirt/config'
@@ -87,6 +89,7 @@ describe 'templates/domain' do
                      target_port: '4242',
                      source_path: '/tmp/foo')
       domain.random(model: 'random')
+      domain.serial(:type => 'file', :source => {:path => '/var/log/vm_consoles/machine.log'})
       domain.pci(bus: '0x06', slot: '0x12', function: '0x5')
       domain.pci(domain: '0x0001', bus: '0x03', slot: '0x00', function: '0x0')
       domain.usb_controller(model: 'nec-xhci', ports: '4')
@@ -109,6 +112,8 @@ describe 'templates/domain' do
       domain.shares = '1024'
       domain.cpuset = '1-4,^3,6'
       domain.nodeset = '1-4,^3,6'
+
+      domain.video_accel3d = true
     end
     let(:test_file) { 'domain_all_settings.xml' }
     it 'renders template' do
